#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com


from blinker import signal

pre_get_instance = signal('bzz.pre-get-instance')
post_get_instance = signal('bzz.post-get-instance')

pre_get_list = signal('bzz.pre-get-list')
post_get_list = signal('bzz.post-get-list')

pre_create_instance = signal('bzz.pre-create-instance')
post_create_instance = signal('bzz.post-create-instance')

pre_update_instance = signal('bzz.pre-update-instance')
post_update_instance = signal('bzz.post-update-instance')

pre_delete_instance = signal('bzz.pre-delete-instance')
post_delete_instance = signal('bzz.post-delete-instance')

authorized_user = signal('bzz.authorized-user')
unauthorized_user = signal('bzz.unauthorized-user')



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of marketplace-v2.
# https://github.com/globoi/marketplacev2


import slugify
import factory
import factory.mongoengine as mongofactory

import tests.models.mongoengine_models as models


class UserFactory(mongofactory.MongoEngineFactory):
    name = factory.Sequence(lambda i: u'user %d' % i)
    slug = factory.LazyAttribute(lambda user: slugify.slugify(user.name))
    email = factory.Sequence(lambda i: u'user-%d@whatever.foo' % i)

    class Meta:
        model = models.User


class OtherUserFactory(mongofactory.MongoEngineFactory):
    name = factory.Sequence(lambda i: u'other-user %d' % i)
    slug = factory.LazyAttribute(lambda user: slugify.slugify(user.name))
    email = factory.Sequence(lambda i: u'other-user-%d@whatever.foo' % i)

    class Meta:
        model = models.OtherUser



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com

from datetime import datetime
from mock import Mock, patch

import cow.server as server
import tornado.testing as testing
import tornado.gen as gen
from tornado.web import RequestHandler
from tornado.httpclient import HTTPError
from preggy import expect
import derpconf.config as config

import bzz
import bzz.signals as signals
import bzz.utils as utils
import tests.base as base
from bzz.providers.google import GoogleProvider


def load_json(json_string):
    try:
        return utils.loads(json_string)
    except ValueError:
        return utils.loads(json_string.decode('utf-8'))


class MockProvider(bzz.AuthProvider):
    @gen.coroutine
    def authenticate(self, access_token):
        raise gen.Return({'id': "123"})


class MockUnauthorizedProvider(bzz.AuthProvider):
    @gen.coroutine
    def authenticate(self, access_token):
        raise gen.Return(None)


class TestAuthHandler(RequestHandler):

    @bzz.authenticated
    def get(self):
        self.write('OK')


class TestServer(server.Server):

    def __init__(self, *args, **kwargs):
        self.io_loop = kwargs.pop('io_loop', None)
        super(TestServer, self).__init__(*args, **kwargs)

    def get_handlers(self):
        handlers_list = bzz.AuthHive.routes_for([
            GoogleProvider(self.io_loop),
            MockUnauthorizedProvider(self.io_loop),
            MockProvider(self.io_loop)
        ])
        handlers_list += [
            ('/test_authentication/', TestAuthHandler)
        ]
        return handlers_list


class TestPrefixServer(server.Server):

    def __init__(self, *args, **kwargs):
        self.io_loop = kwargs.pop('io_loop', None)
        super(TestPrefixServer, self).__init__(*args, **kwargs)

    def get_handlers(self):
        handlers_list = bzz.AuthHive.routes_for([
            GoogleProvider(self.io_loop)
        ], prefix="api")

        return handlers_list


class AuthHiveTestCase(base.ApiTestCase):
    def setUp(self):
        super(AuthHiveTestCase, self).setUp()
        signals.authorized_user.receivers = {}

    def get_config(self):
        return {}

    def get_app(self):
        app = super(AuthHiveTestCase, self).get_app()
        bzz.AuthHive.configure(
            app,
            cookie_name='TEST_AUTH_COOKIE',
            secret_key='TEST_SECRET_KEY',
            expiration=1200
        )
        return app

    def get_server(self):
        cfg = config.Config(**self.get_config())
        self.server = TestServer(config=cfg, io_loop=self.io_loop)
        return self.server

    def mock_auth_cookie(self, user_id, provider, data=None, token='12345', expiration=None):
        if data is None:
            data = {}

        if expiration is None:
            expiration = datetime(year=5000, month=11, day=30)

        jwt = self.server.application.authentication_options['jwt']
        token = jwt.encode({
            'sub': user_id, 'data': data, 'iss': provider, 'token': token,
            'exp': expiration
        })
        cookie_name = self.server.application.authentication_options['cookie_name']
        return '='.join((
            cookie_name, token.decode('utf-8')
        ))

    @testing.gen_test
    def test_cant_authenticate_with_invalid_provider(self):
        try:
            response = yield self.http_client.fetch(
                self.get_url('/auth/signin/'),
                method='POST',
                body=utils.dumps({
                    'access_token': '1234567890', 'provider': 'invalid'
                })
            )
        except HTTPError as e:
            response = e.response

        expect(response.code).to_equal(401)
        expect(response.reason).to_equal('Unauthorized')

    @testing.gen_test
    def test_automagic_provider_name(self):
        class TestProvider(bzz.AuthProvider):
            pass
        instance = TestProvider()
        expect(instance.get_name()).to_equal('test')

    @testing.gen_test
    def test_can_authenticate(self):
        response = yield self.http_client.fetch(
            self.get_url('/auth/signin/'),
            method='POST',
            body=utils.dumps({
                'access_token': '1234567890', 'provider': 'mock'
            })
        )

        expect(response.code).to_equal(200)
        expect(load_json(response.body)).to_be_like(dict(authenticated=True, id="123"))
        cookie_name = self.server.application.authentication_options['cookie_name']
        expect(cookie_name in response.headers.get('Set-Cookie')).to_equal(True)

    @testing.gen_test
    def test_cant_authenticate(self):
        try:
            yield self.http_client.fetch(
                self.get_url('/auth/signin/'),
                method='POST',
                body=utils.dumps({
                    'access_token': '1234567890', 'provider': 'mockunauthorized'
                })
            )
        except HTTPError as e:
            expect(e.response.code).to_equal(401)
            expect(e.response.reason).to_equal('Unauthorized')
            expect(e.response.headers.get('Cookie')).to_equal(None)
            expect(e.response.headers.get('Set-Cookie')).to_equal(None)
        else:
            assert False, 'Should not get this far'

    @testing.gen_test
    def test_can_check_authenticated_request(self):
        response = yield self.http_client.fetch(
            self.get_url('/auth/me/'),
            headers={'Cookie': self.mock_auth_cookie(0, 'mock', data={'id': 0})}
        )

        expect(response.code).to_equal(200)
        expect(load_json(response.body)).to_equal(dict(
            authenticated=True, userData={'id': 0}
        ))

    @testing.gen_test
    def test_can_check_not_authenticated_request(self):
        response = yield self.http_client.fetch(
            self.get_url('/auth/me/')
        )

        expect(response.code).to_equal(200)
        expect(load_json(response.body)).to_equal(dict(authenticated=False))

    @testing.gen_test
    def test_cannot_authenticate_a_user_with_invalid_google_plus_token(self):
        with patch.object(GoogleProvider, '_fetch_userinfo') as provider_mock:
            result = gen.Future()
            response_mock = Mock(code=401, reason='Unauthorized')
            result.set_result(response_mock)
            provider_mock.return_value = result
            try:
                response = yield self.http_client.fetch(
                    self.get_url('/auth/signin/'), method='POST',
                    body=utils.dumps({
                        'access_token': 'INVALID-TOKEN', 'provider': 'google'
                    })
                )
            except HTTPError as e:
                response = e.response
            expect(response.code).to_equal(401)
            expect(response.reason).to_equal('Unauthorized')

    @testing.gen_test
    def test_can_authenticate_a_user_with_valid_google_plus_token(self):
        with patch.object(GoogleProvider, '_fetch_userinfo') as provider_mock:
            result = gen.Future()
            response_mock = Mock(code=200, body=(
                '{"email":"test@gmail.com", "name":"teste", "id":"56789"}'
            ))
            result.set_result(response_mock)
            provider_mock.return_value = result
            try:
                response = yield self.http_client.fetch(
                    self.get_url('/auth/signin/'), method='POST',
                    body=utils.dumps({
                        'access_token': 'VALID-TOKEN', 'provider': 'google'
                    })
                )
            except HTTPError as e:
                response = e.response

            expect(response.code).to_equal(200)
            expect(utils.loads(response.body)['authenticated']).to_be_true()

    @testing.gen_test
    def test_can_send_signal_on_post_authentication(self):

        test_result = {}

        @signals.authorized_user.connect
        def test_signal(provider, user_data=None):
            test_result['provider'] = 'google'
            expect(provider).to_equal('google')
            expect(user_data).to_equal({
                "authenticated": True,
                "provider": "google",
                "email": "test@gmail.com", "name": "Teste", "id": "56789"
            })
            user_data['something'] = 'else'

        with patch.object(GoogleProvider, '_fetch_userinfo') as provider_mock:
            result = gen.Future()
            response_mock = Mock(code=200, body=(
                '{"email":"test@gmail.com", "name":"Teste", "id":"56789"}'
            ))
            result.set_result(response_mock)
            provider_mock.return_value = result
            response = yield self.http_client.fetch(
                self.get_url('/auth/signin/'), method='POST',
                body=utils.dumps({
                    'access_token': 'VALID-TOKEN', 'provider': 'google'
                })
            )
            expect(response.code).to_equal(200)
            data = utils.loads(response.body)
            expect(data['authenticated']).to_be_true()
            expect(data['something']).to_equal('else')
            expect(test_result).to_include('provider')

    @testing.gen_test
    def test_can_send_signal_on_post_unauthorized(self):

        test_result = {}

        @signals.unauthorized_user.connect
        def test_signal(provider):
            test_result['provider'] = provider

        with patch.object(GoogleProvider, '_fetch_userinfo') as provider_mock:
            result = gen.Future()
            response_mock = Mock(code=401, reason='Unauthorized')
            result.set_result(response_mock)
            provider_mock.return_value = result
            try:
                response = yield self.http_client.fetch(
                    self.get_url('/auth/signin/'), method='POST',
                    body=utils.dumps({
                        'access_token': 'VALID-TOKEN', 'provider': 'google'
                    })
                )
            except HTTPError as e:
                response = e.response
            expect(response.code).to_equal(401)
            expect(test_result).to_include('provider')

    @testing.gen_test
    def test_can_make_a_request_in_a_decorated_method_as_authenticated(self):
        response = yield self.http_client.fetch(
            self.get_url('/test_authentication/'),
            headers={'Cookie': self.mock_auth_cookie(
                user_id=0, provider='mock', data={'id': 0}
            )}
        )

        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

    @testing.gen_test
    def test_cant_make_a_request_in_a_decorated_method_as_anonymous(self):
        try:
            response = yield self.http_client.fetch(
                self.get_url('/test_authentication/'),
            )
        except HTTPError as e:
            response = e.response

        expect(response.code).to_equal(401)
        expect(response.reason).to_equal('Unauthorized')

    @testing.gen_test
    def test_can_signout_when_logged_in(self):
        response = yield self.http_client.fetch(
            self.get_url('/auth/signout/'), method='POST', body='',
            headers={'Cookie': self.mock_auth_cookie(
                user_id=0, provider='mock', data={'id': 0}
            )}
        )

        expect(response.code).to_equal(200)
        expect(utils.loads(response.body)).to_equal({'loggedOut': True})

    @testing.gen_test
    def test_can_signout_when_logged_out(self):
        response = yield self.http_client.fetch(
            self.get_url('/auth/signout/'), method='POST', body='',
        )

        expect(response.code).to_equal(200)
        expect(utils.loads(response.body)).to_equal({'loggedOut': True})


class PrefixedAuthHiveTestCase(base.ApiTestCase):
    def setUp(self):
        super(PrefixedAuthHiveTestCase, self).setUp()
        signals.authorized_user.receivers = {}

    def get_config(self):
        return {}

    def get_app(self):
        app = super(PrefixedAuthHiveTestCase, self).get_app()
        bzz.AuthHive.configure(
            app,
            cookie_name='TEST_AUTH_COOKIE',
            secret_key='TEST_SECRET_KEY',
            expiration=1200
        )
        return app

    def get_server(self):
        cfg = config.Config(**self.get_config())
        self.server = TestPrefixServer(config=cfg, io_loop=self.io_loop)
        return self.server

    @testing.gen_test
    def test_can_authenticate_a_user_with_valid_google_plus_token_with_prefix_in_authentication_url(self):
        with patch.object(GoogleProvider, '_fetch_userinfo') as provider_mock:
            result = gen.Future()
            response_mock = Mock(code=200, body=(
                '{"email":"test@gmail.com", "name":"teste", "id":"56789"}'
            ))
            result.set_result(response_mock)
            provider_mock.return_value = result
            try:
                response = yield self.http_client.fetch(
                    self.get_url('/api/auth/signin/'), method='POST',
                    body=utils.dumps({
                        'access_token': 'VALID-TOKEN', 'provider': 'google'
                    })
                )
            except HTTPError as e:
                response = e.response

            expect(response.code).to_equal(200)
            expect(utils.loads(response.body)['authenticated']).to_be_true()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com

import slugify
import mongoengine


class Address(mongoengine.Document):
    street = mongoengine.StringField(required=True)


class User(mongoengine.Document):
    name = mongoengine.StringField(required=True)
    email = mongoengine.StringField(required=True)
    slug = mongoengine.StringField(required=False)
    addresses = mongoengine.ListField(mongoengine.ReferenceField(Address), required=False)

    def save(self, *args, **kw):
        self.slug = slugify.slugify(self.name, to_lower=True)
        super(User, self).save(*args, **kw)


class OtherUser(mongoengine.Document):
    name = mongoengine.StringField(required=True)
    email = mongoengine.StringField(required=True)
    slug = mongoengine.StringField(required=False)

    def to_dict(self):
        return {
            "user": "%s <%s>" % (self.name, self.email)
        }

    def save(self, *args, **kw):
        self.slug = slugify.slugify(self.name, to_lower=True)
        super(OtherUser, self).save(*args, **kw)

    @classmethod
    def get_id_field_name(cls):
        return OtherUser.slug


class Team(mongoengine.Document):
    name = mongoengine.StringField(required=True)
    users = mongoengine.ListField(mongoengine.ReferenceField(User))


class GrandChild(mongoengine.EmbeddedDocument):
    first_name = mongoengine.StringField(required=True)
    last_name = mongoengine.StringField(required=True)


class Child(mongoengine.EmbeddedDocument):
    first_name = mongoengine.StringField(required=True)
    last_name = mongoengine.StringField(required=True)
    child = mongoengine.EmbeddedDocumentField(GrandChild, required=False)

    @classmethod
    def get_id_field_name(cls):
        return Child.first_name


class Parent(mongoengine.Document):
    name = mongoengine.StringField(required=True)
    child = mongoengine.EmbeddedDocumentField(Child, required=False)


class Parent2(mongoengine.Document):
    name = mongoengine.StringField(required=True)
    children = mongoengine.ListField(mongoengine.EmbeddedDocumentField(Child))


class Person(mongoengine.Document):
    name = mongoengine.StringField(required=True)


class Student(mongoengine.Document):
    code = mongoengine.StringField(required=True)
    person = mongoengine.ReferenceField(Person)


class CustomQuerySet(mongoengine.Document):
    prop = mongoengine.StringField()
    meta = {'collection': 'custom_queryset'}

    @classmethod
    def get_list_queryset(cls, queryset, handler):
        return queryset(prop='Bernardo Heynemann')

    @classmethod
    def get_instance_queryset(cls, model, queryset, instance_id, handler):
        return queryset.filter(prop='Bernardo Heynemann')


class UniqueUser(mongoengine.Document):
    name = mongoengine.StringField(unique=True)
    meta = {'collection': 'unique_user'}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com

from datetime import datetime
import types

import mongoengine as me
import cow.server as server
import cow.plugins.mongoengine_plugin as mongoengine_plugin
from preggy import expect
import derpconf.config as config
import tornado.web

import bzz
import bzz.signals as signals
import bzz.utils as utils
import tests.base as base


RESPONSE_400 = '<html><title>400:badrequest</title><body>400:badrequest</body></html>'


class MongoEngineEndToEndTestCase(base.ApiTestCase):
    def __get_test_data(self):
        to_json = lambda body: load_json(body)

        return [
            ('GET', '/user', {}, 200, to_json, []),
            ('POST', '/user', dict(body="name=test%20user&age=32"), 200, None, 'OK'),
            ('GET', '/user', {}, 200, to_json, self.__assert_user_data(name="test user", age=32)),
            ('GET', '/user/test%20user', {}, 200, to_json, self.__assert_user_data(name="test user", age=32)),
            ('PUT', '/user/test%20user', dict(body="age=31"), 200, None, 'OK'),
            ('GET', '/user/test%20user', {}, 200, to_json, self.__assert_user_data(name="test user", age=31)),
            ('POST', '/user', dict(body="name=test-user2&age=32"), 200, None, 'OK'),
            ('DELETE', '/user/test-user2', {}, 200, None, 'OK'),
            ('GET', '/user', {}, 200, to_json, self.__assert_len(1)),
            ('GET', '/team', {}, 200, to_json, []),
            ('POST', '/team', dict(body="code=team-1&owner=test%20user"), 200, None, 'OK'),
            ('GET', '/team/team-1', {}, 200, to_json, self.__assert_team_data(name="team-1", owner="test user")),
            ('POST', '/user', dict(body="name=test-user3&age=32"), 200, None, 'OK'),
            ('PUT', '/team/team-1', dict(body="owner=test-user3"), 200, None, 'OK'),
            ('PUT', '/team/team-1', dict(body="members[]=test-user3"), 400, None, RESPONSE_400),
            ('GET', '/team/team-1', {}, 200, to_json, self.__assert_team_data(name='team-1', member_count=0)),
            ('POST', '/team', dict(body="code=team-2&owner=test%20user&members[]=test%20user"), 200, None, 'OK'),
            ('GET', '/team/team-2', {}, 200, to_json, self.__assert_team_data(name='team-2', member_count=1)),
            ('POST', '/team', dict(body="code=team-3&owner=test%20user&members[]=test%20user&members[]=test-user3"),
                200, None, 'OK'),
            ('GET', '/team/team-3', {}, 200, to_json, self.__assert_team_data(name='team-3', member_count=2)),
            ('DELETE', '/team/team-2', {}, 200, None, 'OK'),
            ('DELETE', '/team/team-3', {}, 200, None, 'OK'),
            ('GET', '/team', {}, 200, to_json, self.__assert_len(1)),
            ('POST', '/team/team-1/members', dict(body="members[]=test%20user"), 200, None, 'OK'),
            ('GET', '/team/team-1/members', {}, 200, to_json, self.__assert_len(1)),
            ('POST', '/user', dict(body="name=test-user4&age=32"), 200, None, 'OK'),
            ('POST', '/team/team-1/members', dict(body="members[]=test-user4"), 200, None, 'OK'),
            ('DELETE', '/team/team-1/members/test-user4', {}, 200, None, 'OK'),
            ('PUT', '/team/team-1/members/test-user4', dict(body=""), 400, None, RESPONSE_400),
            ('GET', '/team/team-1/members', {}, 200, to_json, self.__assert_len(1)),
            ('GET', '/user/test-user4', {}, 200, to_json, self.__assert_user_data(name="test-user4", age=32)),
            ('POST', '/team/team-1/projects', dict(body="name=project-1&module.name=module-name"), 200, None, 'OK'),
            ('GET', '/team/team-1/projects', {}, 200, to_json, self.__assert_len(1)),
            ('GET', '/team/team-1/projects/project-1', {}, 200, to_json,
                self.__assert_project_data(name="project-1", module="module-name")),
            ('GET', '/team/team-1/projects/project-1/module', {}, 200, to_json, self.__assert_module_data(name="module-name")),
        ]

    def test_end_to_end_flow(self):
        data = self.__get_test_data()

        print("")
        print("")
        print("")
        print("Doing end-to-end test:")
        print("")
        for url_arguments in data:
            self.validate_request(url_arguments)
        print("")

    def setUp(self):
        super(MongoEngineEndToEndTestCase, self).setUp()
        signals.post_create_instance.receivers = {}
        signals.post_update_instance.receivers = {}
        signals.post_delete_instance.receivers = {}
        EndToEndUser.objects.delete()
        EndToEndTeam.objects.delete()

    def get_config(self):
        return dict(
            MONGO_DATABASES={
                'default': {
                    'host': 'localhost',
                    'port': 3334,
                    'database': 'bzz_test'
                }
            },
        )

    def get_server(self):
        cfg = config.Config(**self.get_config())
        self.server = TestServer(config=cfg)
        return self.server

    def __assert_project_data(self, name=None, module=None):
        def handle(obj):
            if name is not None:
                expect(obj['name']).to_be_like(name)

            if module is not None:
                expect(obj['module']).not_to_be_null()
                expect(obj['module']['name']).to_be_like(module)

        return handle

    def __assert_module_data(self, name=None):
        def handle(obj):
            if name is not None:
                expect(obj['name']).to_be_like(name)

        return handle

    def __assert_user_data(self, created_at=None, age=None, id_=None, name=None):
        def handle(obj):
            if isinstance(obj, (list, tuple)):
                obj = obj[0]

            if created_at is not None:
                expect(obj['created_at']).to_be_like(created_at)

            if age is not None:
                expect(obj['age']).to_equal(age)

            if id_ is not None:
                expect(obj['id']).to_be_like(id_)

            if name is not None:
                expect(obj['name']).to_equal(name)

        return handle

    def __assert_team_data(self, owner=None, name=None, member_count=None):
        def handle(obj):
            if isinstance(obj, (list, tuple)):
                obj = obj[0]

            if owner is not None:
                expect(obj['owner']).not_to_be_null()
                expect(obj['owner']['name']).to_equal(owner)

            if name is not None:
                expect(obj['code']).to_equal(name)

            if member_count is not None:
                expect(obj['members']).to_length(member_count)

        return handle

    def __assert_len(self, expected_length):
        def handle(obj):
            expect(obj).not_to_be_null()
            expect(obj).to_be_instance_of(list)
            expect(obj).to_length(expected_length)
        return handle

    def validate_request(self, url_arguments):
        method, url, options, expected_status_code, transform_body, expected_body = url_arguments

        print("B %s %s..." % (method, url))
        self.http_client.fetch(
            self.get_url(url),
            method=method,
            callback=self.stop,
            **options
        )
        response = self.wait()
        expect(response.code).to_equal(expected_status_code)

        body = response.body
        if transform_body is not None:
            body = transform_body(response.body)

        if isinstance(expected_body, types.FunctionType):
            expected_body(body)
        else:
            expect(body).to_be_like(expected_body)

        print("A %s %s - %s" % (method, url, response.code))


def load_json(json_string):
    try:
        return utils.loads(json_string)
    except ValueError:
        return utils.loads(json_string.decode('utf-8'))


class NamedEmbeddedDocument(me.EmbeddedDocument):
    meta = {'allow_inheritance': True}

    name = me.StringField()


class EndToEndUser(me.Document):
    meta = {'collection': 'EndToEndUser'}

    name = me.StringField()
    age = me.IntField()
    created_at = me.DateTimeField(default=datetime.now)

    @classmethod
    def get_id_field_name(self):
        return EndToEndUser.name


class EndToEndTeam(me.Document):
    meta = {'collection': 'EndToEndTeam'}

    code = me.StringField()
    owner = me.ReferenceField(EndToEndUser)
    members = me.ListField(me.ReferenceField(EndToEndUser))
    projects = me.ListField(me.EmbeddedDocumentField("Project"))

    @classmethod
    def get_id_field_name(self):
        return EndToEndTeam.code

    def to_dict(self):
        return {
            'code': self.code,
            'owner': {
                'name': self.owner.name
            },
            'members': [member.name for member in self.members],
            'projects': self.projects
        }


class Project(NamedEmbeddedDocument):
    module = me.EmbeddedDocumentField("Module")

    @classmethod
    def get_id_field_name(self):
        return Project.name


class Module(NamedEmbeddedDocument):
    @classmethod
    def get_id_field_name(self):
        return Module.name


class VersionHandler(tornado.web.RequestHandler):
    def get(self):
        self.write(bzz.__version__)


class TestServer(server.Server):
    def get_plugins(self):
        return [
            mongoengine_plugin.MongoEnginePlugin
        ]

    def get_handlers(self):
        routes = [
            bzz.ModelHive.routes_for('mongoengine', EndToEndUser, resource_name="user"),
            bzz.ModelHive.routes_for('mongoengine', EndToEndTeam, resource_name="team"),
            ('/version', VersionHandler),
        ]
        return bzz.flatten(routes)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com

import inspect
import re
import calendar
import datetime
from six.moves import reduce

import jwt

import bzz.core as core

try:
    import ujson as json
    UJSON_ENABLED = True
except ImportError:
    UJSON_ENABLED = False
    import json


first_cap_re = re.compile('(.)([A-Z][a-z]+)')
all_cap_re = re.compile('([a-z0-9])([A-Z])')


def flatten(routes):
    '''
    Gets a list of routes that includes hive-generated routes (model, auth or mock), as well as
    user created routes and flatten the list to the format tornado expects.

    Can be used in bzz namespace::

        import bzz
        bzz.flatten(routes)

    :param routes: list of routes created by bzz hives or by the user
    :returns: List of routes that a tornado app expects.
    '''
    result = []
    for route_list in routes:
        if isinstance(route_list, core.RouteList):
            for route in route_list:
                result.append(route)
        else:
            result.append(route_list)

    return tuple(result)


def convert(name):
    s1 = first_cap_re.sub(r'\1_\2', name)
    return all_cap_re.sub(r'\1_\2', s1).lower()


def get_class(klass):
    module_name, class_name = klass.rsplit('.', 1)

    module = __import__(module_name)

    if '.' in module_name:
        module = reduce(getattr, module_name.split('.')[1:], module)

    return getattr(module, class_name)


def default(obj):
    """Default JSON serializer."""

    if isinstance(obj, datetime.datetime):
        if obj.utcoffset() is not None:
            obj = obj - obj.utcoffset()
    millis = int(
        calendar.timegm(obj.timetuple()) * 1000 +
        obj.microsecond / 1000
    )
    return millis


def loads(data):
    return json.loads(data)


def dumps(instance):
    if UJSON_ENABLED:
        return json.dumps(instance)

    return json.dumps(instance, default=default)

def get_prefix(prefix):
    if not prefix:
        return ''

    return prefix.strip('/')

def add_prefix(prefix, route):
    prefix = get_prefix(prefix)
    route = route.lstrip('/')

    if not prefix:
        return "/%s" % route

    return "/%s/%s" % (prefix, route)

def ensure_instance(provider):
    return provider() if inspect.isclass(provider) else provider


class Jwt(object):
    '''Json Web Tokens encoding/decoding utility class.
    Usage:
    >>> now = datetime.now()
    >>> tokenizer = Jwt('SECRET')
    >>> payload = dict(sub='user@email.com', iss='provider', token='123456789', iat=now, exp=120)
    >>> token = tokenizer.encode(payload)
    >>> tokenizer.decode(token)
    {'sub':'user@email.com', 'iss':'provider', 'token':'123456789',
    'iat': <datetime>, 'exp': <datetime>}
    >>> tokenizer.try_to_decode('invalid-token')
    (False, None)
    '''

    def __init__(self, secret, algo='HS512'):
        self.secret = secret
        self.algo = algo

    def encode(self, payload):
        '''Encodes the payload returning a Json Web Token
        '''
        return jwt.encode(payload, self.secret, self.algo)

    def decode(self, encrypted_payload):
        '''Decodes the Json Web Token returning the payload
        '''
        return jwt.decode(encrypted_payload, self.secret)

    def try_to_decode(self, encrypted_payload):
        '''Tries to decrypt the given encrypted and returns a tuple with
        a decrypted boolean flag and the decrypted object if success is True
        '''
        try:
            return True, self.decode(encrypted_payload)
        except (jwt.ExpiredSignature, jwt.DecodeError, AttributeError):
            return False, None



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com

import sys

from bzz.version import __version__  # NOQA
try:
    from bzz.model import ModelHive  # NOQA
    from bzz.mock import MockHive  # NOQA
    from bzz.auth import AuthHive, AuthProvider, authenticated  # NOQA
    from bzz.utils import flatten  # NOQA
except ImportError:
    err = sys.exc_info()[1]
    print("%s. Probably setup.py" % err)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com

'''
The bzz framework gives you a AuthHive class to allow easy OAuth2 authentication with a few steps.
'''

import functools
from datetime import datetime, timedelta

import tornado.web
import tornado.gen as gen
from tornado import ioloop
from tornado import httpclient

import bzz.signals as signals
import bzz.utils as utils
import bzz.core as core


def authenticated(method):
    '''Decorate methods with this to require the user to be authenticated.

    If the user is not logged in (cookie token expired, invalid or no token),
    a 401 unauthorized status code will be returned.

    If the user is authenticated, the token cookie will be renewed
    with more `expiration` seconds (configured in `AuthHive.configure` method).

    Usage:

    .. testcode:: auth_example_2

        import tornado
        import bzz

        class MyHandler(tornado.web.RequestHandler):

            @bzz.authenticated
            def get(self):
                self.write('I`m authenticated! :)')
    '''
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        authenticated, payload = AuthHandler.is_authenticated(self)
        if authenticated:
            AuthHandler._renew_authentication(self, payload)
        else:
            AuthHandler._set_unauthorized(self)
        return method(self, *args, **kwargs)
    return wrapper


class AuthHive(object):
    '''
    The AuthHive is the responsible for integrating authentication into your API.
    '''

    @classmethod
    def configure(cls, app, secret_key, expiration=1200, cookie_name='AUTH_TOKEN'):
        '''Configure the application to the authentication ecosystem.

        :param app: The tornado application to configure
        :type app: tornado.web.Application instance
        :param secret_key: A string to use for encoding/decoding Jwt that must
                           be private
        :type secret_key: str
        :param expiration: Time in seconds to the expiration (time to live) of
                           the token
        :type expiration: int
        :param cookie_name: The name of the cookie
        :type cookie_name: str

        '''
        app.authentication_options = {
            'secret_key': secret_key,
            'expiration': expiration,
            'cookie_name': cookie_name,
            'jwt': utils.Jwt(secret_key)
        }

    @classmethod
    def routes_for(cls, providers, prefix=''):
        '''Returns the list of routes for the authentication ecosystem with the
        given providers configured.

        The routes returned are for these URLs:

        * [prefix]/auth/me/ -- To get user data and check if authenticated
        * [prefix]/auth/signin/ -- To sign in using the specified provider
        * [prefix]/auth/signout/ -- To sign out using the specified provider

        :param providers: A list of providers
        :type providers: AuthProvider class or instance
        :param prefix: An optional argument that can be specified as means to include a prefix route (i.e.: '/api');
        :type prefix: String
        :returns: list of tornado routes (url, handler, initializers)
        '''
        options = {
            'providers': dict([
                (provider.get_name(), utils.ensure_instance(provider))
                for provider in providers
            ])
        }

        url = functools.partial(utils.add_prefix, prefix)

        return core.RouteList([
            (url('/auth/me/'), AuthMeHandler, options),
            (url('/auth/signin/'), AuthSigninHandler, options),
            (url('/auth/signout/'), AuthSignoutHandler, options),
        ])


class AuthHandler(tornado.web.RequestHandler):

    def initialize(self, providers):
        self.providers = providers
        self.jwt = self.application.authentication_options['jwt']
        self.expiration = self.application.authentication_options['expiration']
        self.cookie_name = self.application.authentication_options['cookie_name']

    @classmethod
    def _set_unauthorized(cls, handler):
        handler.set_status(401, reason='Unauthorized')
        raise tornado.web.Finish()

    @classmethod
    def is_authenticated(cls, handler):
        jwt = handler.application.authentication_options['jwt']
        cookie_name = handler.application.authentication_options['cookie_name']
        return jwt.try_to_decode(handler.get_cookie(cookie_name))

    @classmethod
    def _renew_authentication(cls, handler, payload):
        payload.update(dict(
            iat=datetime.utcnow(),
            exp=datetime.utcnow() + timedelta(
                seconds=handler.application.authentication_options['expiration']
            )
        ))
        cookie_name = handler.application.authentication_options['cookie_name']
        jwt = handler.application.authentication_options['jwt']
        token = jwt.encode(payload)
        handler.set_cookie(cookie_name, token)


class AuthMeHandler(AuthHandler):

    def get(self):
        '''
        Returns if request is authenticated, if is, returns user`s data too.
        '''
        authenticated, payload = AuthHandler.is_authenticated(self)
        result = dict(authenticated=authenticated)
        if authenticated:
            result['userData'] = payload['data']
        self.set_status(200)
        self.write(result)


class AuthSigninHandler(AuthHandler):

    @gen.coroutine
    def post(self):
        '''
        Try to authenticate user with the access_token POST data.
        If the `self.authenticate` method returns the user, create a JSON
        Web Token (JWT) and set a `cookie_name` cookie with the encoded
        value. Otherwise returns a unauthorized request.
        '''
        post_data = utils.loads(self.request.body)
        access_token = post_data.get('access_token')
        provider_name = post_data.get('provider')

        provider = self.providers.get(provider_name, None)
        if provider is None:
            AuthHandler._set_unauthorized(self)

        user_data = yield provider.authenticate(access_token)
        if user_data:
            payload = dict(
                sub=user_data['id'],
                data=user_data,
                iss=provider_name,
                token=access_token,
                iat=datetime.utcnow(),
                exp=datetime.utcnow() + timedelta(seconds=self.expiration)
            )
            auth_token = self.jwt.encode(payload)

            user_data['authenticated'] = True
            signals.authorized_user.send(provider_name, user_data=user_data)
            self.set_cookie(self.cookie_name, auth_token)
            self.write(user_data)
        else:
            signals.unauthorized_user.send(provider_name)
            AuthHandler._set_unauthorized(self)


class AuthSignoutHandler(AuthHandler):

    def post(self):
        self.clear_cookie(self.cookie_name)
        self.write({'loggedOut': True})


class AuthProvider(object):

    def __init__(self, io_loop=None):
        if not io_loop:
            io_loop = ioloop.IOLoop.instance()
        self.http_client = httpclient.AsyncHTTPClient(io_loop=io_loop)

    @classmethod
    def get_name(cls):
        '''Returns the lowercase class name without `Provider`'''
        return cls.__name__.split('Provider')[0].lower()

    @gen.coroutine
    def authenticate(self, access_token):
        raise NotImplementedError('Provider.authenticate method must be implemented')



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com

import unittest as unit
import cow.testing as testing
import cow.server as server
import cow.plugins.mongoengine_plugin as mongoengine_plugin
import derpconf.config as config


class TestCase(unit.TestCase):
    pass


class TestServer(server.Server):
    def get_plugins(self):
        return [
            mongoengine_plugin.MongoEnginePlugin
        ]

    def get_handlers(self):
        routes = []
        return routes


class ApiTestCase(testing.CowTestCase, TestCase):
    def get_config(self):
        return dict(
            MONGO_DATABASES={
                'default': {
                    'host': 'localhost',
                    'port': 3334,
                    'database': 'bzz_test'
                }
            },
        )

    def get_server(self):
        cfg = config.Config(**self.get_config())
        self.server = TestServer(config=cfg)
        return self.server



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com

import mongoengine
import cow.server as server
import cow.plugins.mongoengine_plugin as mongoengine_plugin
import tornado.testing as testing
from tornado.httpclient import HTTPError
from preggy import expect
import derpconf.config as config
import bson.objectid as oid

import bzz
import bzz.providers.mongoengine_provider as me
import bzz.signals as signals
import bzz.utils as utils
import tests.base as base
import tests.models.mongoengine_models as models
import tests.fixtures as fix


def load_json(json_string):
    try:
        return utils.loads(json_string)
    except ValueError:
        return utils.loads(json_string.decode('utf-8'))


class TestServer(server.Server):
    def get_plugins(self):
        return [
            mongoengine_plugin.MongoEnginePlugin
        ]

    def get_handlers(self):
        routes = [
            bzz.ModelHive.routes_for('mongoengine', models.User),
            bzz.ModelHive.routes_for('mongoengine', models.OtherUser),
            bzz.ModelHive.routes_for('mongoengine', models.Parent),
            bzz.ModelHive.routes_for('mongoengine', models.Parent2),
            bzz.ModelHive.routes_for('mongoengine', models.Team),
            bzz.ModelHive.routes_for('mongoengine', models.Student),
            bzz.ModelHive.routes_for('mongoengine', models.CustomQuerySet),
        ]
        return bzz.flatten(routes)


class MongoEngineProviderTestCase(base.ApiTestCase):
    def setUp(self):
        super(MongoEngineProviderTestCase, self).setUp()
        signals.pre_get_instance.receivers = {}
        signals.post_get_instance.receivers = {}
        signals.pre_get_list.receivers = {}
        signals.post_get_list.receivers = {}
        signals.pre_create_instance.receivers = {}
        signals.post_create_instance.receivers = {}
        signals.pre_update_instance.receivers = {}
        signals.post_update_instance.receivers = {}
        signals.pre_delete_instance.receivers = {}
        signals.post_delete_instance.receivers = {}

        models.User.objects.delete()
        models.OtherUser.objects.delete()
        models.Parent.objects.delete()
        models.Parent2.objects.delete()
        models.Team.objects.delete()
        models.Student.objects.delete()

    def get_config(self):
        return dict(
            MONGO_DATABASES={
                'default': {
                    'host': 'localhost',
                    'port': 3334,
                    'database': 'bzz_test'
                }
            },
        )

    def get_server(self):
        cfg = config.Config(**self.get_config())
        self.server = TestServer(config=cfg)
        return self.server

    @testing.gen_test
    def test_can_create_user(self):
        response = yield self.http_client.fetch(
            self.get_url('/user/'),
            method='POST',
            body='name=Bernardo%20Heynemann&email=heynemann@gmail.com'
        )

        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')
        expect(response.headers).to_include('X-Created-Id')
        expect(response.headers).to_include('location')

        expected_url = '/user/%s/' % response.headers['X-Created-Id']
        expect(response.headers['location']).to_equal(expected_url)

    @testing.gen_test
    def test_can_get_user(self):
        user = fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user/%s' % user.id),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj['email']).to_equal(user.email)
        expect(obj['name']).to_equal(user.name)
        expect(obj['slug']).to_equal(user.slug)

    @testing.gen_test
    def test_getting_invalid_user_fails_with_403(self):
        objectid = oid.ObjectId()
        err = expect.error_to_happen(HTTPError)
        with err:
            yield self.http_client.fetch(
                self.get_url('/user/%s' % objectid)
            )
        expect(err.error.code).to_equal(404)

    @testing.gen_test
    def test_can_create_other_user(self):
        response = yield self.http_client.fetch(
            self.get_url('/other_user/'),
            method='POST',
            body='name=Bernardo%20Heynemann&email=heynemann@gmail.com'
        )

        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')
        expect(response.headers).to_include('X-Created-Id')
        expect(response.headers['X-Created-Id']).to_equal('bernardo-heynemann')
        expect(response.headers).to_include('location')

        expected_url = '/other_user/%s/' % response.headers['X-Created-Id']
        expect(response.headers['location']).to_equal(expected_url)

    @testing.gen_test
    def test_can_get_other_user(self):
        user = fix.OtherUserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/other_user/%s' % user.slug),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj['user']).to_equal('%s <%s>' % (user.name, user.email))

    @testing.gen_test
    def test_can_get_list(self):
        models.User.objects.delete()
        for i in range(30):
            fix.UserFactory.create()

        response = yield self.http_client.fetch(
            self.get_url('/user/'),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj).to_length(20)

        response = yield self.http_client.fetch(
            self.get_url('/user/?page=2'),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj).to_length(10)

        response = yield self.http_client.fetch(
            self.get_url('/user/?page=3'),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj).to_length(10)

        response = yield self.http_client.fetch(
            self.get_url('/user/?page=qwe'),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj).to_length(20)

    @testing.gen_test
    def test_can_get_filtered_list(self):
        models.User.objects.delete()
        users = []
        for i in range(30):
            users.append(fix.UserFactory.create())

        response = yield self.http_client.fetch(
            self.get_url('/user/?slug=%s' % users[0].slug),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj).to_length(1)

    @testing.gen_test
    def test_can_update(self):
        user = fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user/%s' % user.id),
            method='PUT',
            headers={
                "Content-Type": "application/x-www-form-urlencoded"
            },
            body='name=Rafael%20Floriano&email=rflorianobr@gmail.com'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        loaded_user = models.User.objects.get(id=user.id)
        expect(loaded_user.name).to_equal('Rafael Floriano')
        expect(loaded_user.slug).to_equal('rafael-floriano')
        expect(loaded_user.email).to_equal('rflorianobr@gmail.com')

    @testing.gen_test
    def test_can_delete(self):
        user = fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user/%s' % user.id),
            method='DELETE'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        with expect.error_to_happen(mongoengine.errors.DoesNotExist):
            models.User.objects.get(id=user.id)

    @testing.gen_test
    def test_can_delete_not_found_instance(self):
        response = yield self.http_client.fetch(
            self.get_url('/other_user/invalid'),
            method='DELETE'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('FAIL')

    @testing.gen_test
    def test_can_subscribe_to_pre_create_signal(self):
        instances = []

        def handle_pre_create(sender, arguments, handler):
            instances.append((arguments, handler))

        signals.pre_create_instance.connect(handle_pre_create)

        response = yield self.http_client.fetch(
            self.get_url('/user/'),
            method='POST',
            body='name=Bernardo%20Heynemann&email=heynemann@gmail.com'
        )

        expect(response.code).to_equal(200)
        expect(instances).to_length(1)
        expect(instances[0][0]).to_be_like(['user'])

    @testing.gen_test
    def test_can_subscribe_to_pre_create_signal_on_internal_urls(self):
        instances = []

        def handle_pre_create(sender, arguments, handler):
            instances.append((sender, arguments, handler))

        signals.pre_create_instance.connect(handle_pre_create)

        user = models.User(name="Bernardo Heynemann", email="foo@bar.com")
        user.save()
        team = models.Team(name="test-team", users=[user])
        team.save()

        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users/' % team.id),
            method='POST',
            body='users[]=%s' % user.id
        )

        expect(response.code).to_equal(200)
        expect(instances).to_length(1)
        expect(instances[0][0]).to_equal(models.User)
        expect(instances[0][1]).to_be_like(['team/%s' % team.id, 'users'])

    @testing.gen_test
    def test_can_subscribe_to_post_create_signal(self):
        instances = {}

        def handle_post_create(sender, instance, handler):
            instances[instance.slug] = instance

        signals.post_create_instance.connect(handle_post_create)

        response = yield self.http_client.fetch(
            self.get_url('/user/'),
            method='POST',
            body='name=Bernardo%20Heynemann&email=heynemann@gmail.com'
        )

        expect(response.code).to_equal(200)
        expect(instances).to_include('bernardo-heynemann')

    @testing.gen_test
    def test_can_subscribe_to_post_create_signal_on_internal_urls(self):
        instances = {}

        def handle_post_create(sender, instance, handler):
            instances[instance.slug] = (sender, instance)

        signals.post_create_instance.connect(handle_post_create)

        user = models.User(name="Bernardo Heynemann", email="foo@bar.com")
        user.save()
        team = models.Team(name="test-team", users=[user])
        team.save()

        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users/' % team.id),
            method='POST',
            body='users[]=%s' % user.id
        )

        expect(response.code).to_equal(200)
        expect(instances).to_include('bernardo-heynemann')
        expect(instances['bernardo-heynemann'][0]).to_equal(models.User)

    @testing.gen_test
    def test_can_subscribe_to_pre_update_signal(self):
        instances = []

        def handle_pre_update(sender, arguments, handler):
            instances.append((arguments, handler))

        signals.pre_update_instance.connect(handle_pre_update)

        user = fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user/%s' % user.id),
            method='PUT',
            headers={
                "Content-Type": "application/x-www-form-urlencoded"
            },
            body='name=Rafael%20Floriano&email=rflorianobr@gmail.com'
        )

        expect(response.code).to_equal(200)
        expect(instances).to_length(1)
        expect(instances[0][0]).to_be_like(['user/%s' % user.id])

    @testing.gen_test
    def test_can_subscribe_to_post_update_signal(self):
        instances = {}
        updated = {}

        def handle_post_update(sender, instance, updated_fields, handler):
            instances[instance.slug] = instance
            updated[instance.slug] = updated_fields

        signals.post_update_instance.connect(handle_post_update)

        user = fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user/%s' % user.id),
            method='PUT',
            headers={
                "Content-Type": "application/x-www-form-urlencoded"
            },
            body='name=Rafael%20Floriano&email=rflorianobr@gmail.com'
        )
        expect(response.code).to_equal(200)

        expect(instances).to_include('rafael-floriano')
        expect(updated).to_include('rafael-floriano')
        expect(updated['rafael-floriano']).to_be_like({
            'name': {
                'from': user.name,
                'to': 'Rafael Floriano'
            },
            'email': {
                'from': user.email,
                'to': 'rflorianobr@gmail.com'
            }
        })

    @testing.gen_test
    def test_can_subscribe_to_pre_delete_signal(self):
        instances = []

        def handle_pre_delete(sender, arguments, handler):
            instances.append((arguments, handler))

        signals.pre_delete_instance.connect(handle_pre_delete)

        user = fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user/%s' % user.id),
            method='DELETE'
        )
        expect(response.code).to_equal(200)
        expect(instances).to_length(1)
        expect(instances[0][0]).to_be_like(['user/%s' % user.id])

    @testing.gen_test
    def test_can_subscribe_to_post_delete_signal(self):
        instances = {}

        def handle_post_delete(sender, instance, handler):
            instances[instance.slug] = instance

        signals.post_delete_instance.connect(handle_post_delete)

        user = fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user/%s' % user.id),
            method='DELETE'
        )
        expect(response.code).to_equal(200)
        expect(instances).to_include(user.slug)

    @testing.gen_test
    def test_can_subscribe_to_pre_get_instance_signal(self):
        instances = []

        def handle_pre_get_instance(sender, arguments, handler):
            instances.append((sender, arguments, handler))

        signals.pre_get_instance.connect(handle_pre_get_instance)

        user = fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user/%s' % user.id),
        )
        expect(response.code).to_equal(200)
        expect(instances).to_length(1)
        expect(instances[0][0]).to_equal(models.User)
        expect(instances[0][1]).to_be_like(['user/%s' % user.id])

    @testing.gen_test
    def test_can_subscribe_to_pre_get_instance_signal_for_internal_classes(self):
        instances = []

        def handle_pre_get_instance(sender, arguments, handler):
            instances.append((sender, arguments, handler))

        signals.pre_get_instance.connect(handle_pre_get_instance)

        user = models.User(name="Bernardo Heynemann", email="foo@bar.com")
        user.save()
        team = models.Team(name="test-team", users=[user])
        team.save()

        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users/%s' % (str(team.id), str(user.id))),
        )

        expect(response.code).to_equal(200)
        expect(instances).to_length(1)
        expect(instances[0][0]).to_equal(models.User)
        expect(instances[0][1]).to_be_like(['team/%s' % team.id, 'users/%s' % user.id])

    @testing.gen_test
    def test_can_subscribe_to_post_get_instance_signal(self):
        instances = {}

        def handle_post_get_instance(sender, instance, handler):
            instances[instance.slug] = instance

        signals.post_get_instance.connect(handle_post_get_instance)

        user = fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user/%s' % user.id),
        )
        expect(response.code).to_equal(200)
        expect(instances).to_include(user.slug)

    @testing.gen_test
    def test_can_subscribe_to_post_get_instance_signal_for_internal_urls(self):
        instances = {}

        def handle_post_get_instance(sender, instance, handler):
            instances[instance.slug] = (sender, instance)

        signals.post_get_instance.connect(handle_post_get_instance)

        user = models.User(name="Bernardo Heynemann", email="foo@bar.com")
        user.save()
        team = models.Team(name="test-team", users=[user])
        team.save()

        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users/%s' % (str(team.id), str(user.id))),
        )
        expect(response.code).to_equal(200)
        expect(instances).to_length(1)
        expect(instances).to_include(user.slug)
        expect(instances[user.slug][0]).to_equal(models.User)
        expect(instances[user.slug][1].id).to_equal(user.id)

    @testing.gen_test
    def test_can_subscribe_to_pre_get_list_signal(self):
        lists = []

        def handle_pre_get_list(sender, arguments, handler):
            lists.append((arguments, handler))

        signals.pre_get_list.connect(handle_pre_get_list)

        fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user'),
        )
        expect(response.code).to_equal(200)
        expect(lists).to_length(1)
        expect(lists[0][0]).to_be_like(['user'])

    @testing.gen_test
    def test_can_subscribe_to_pre_get_list_signal_for_internal_classes(self):
        lists = []

        def handle_pre_get_list(sender, arguments, handler):
            lists.append((sender, arguments, handler))

        signals.pre_get_list.connect(handle_pre_get_list)

        user = models.User(name="Bernardo Heynemann", email="foo@bar.com")
        user.save()
        team = models.Team(name="test-team", users=[user])
        team.save()

        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users' % str(team.id)),
        )

        expect(response.code).to_equal(200)
        expect(lists).to_length(1)
        expect(lists[0][0]).to_equal(models.User)
        expect(lists[0][1]).to_be_like(['team/%s' % team.id, 'users'])

    @testing.gen_test
    def test_can_subscribe_to_post_get_list_signal(self):
        lists = []

        def handle_post_get_list(sender, items, handler):
            lists.append(items)

        signals.post_get_list.connect(handle_post_get_list)

        user = fix.UserFactory.create()
        response = yield self.http_client.fetch(
            self.get_url('/user'),
        )
        expect(response.code).to_equal(200)
        expect(lists).to_length(1)
        expect(lists[0]).to_length(1)
        expect(lists[0][0].slug).to_equal(user.slug)

    @testing.gen_test
    def test_can_subscribe_to_post_get_list_signal_for_internal_urls(self):
        lists = []

        def handle_post_get_list(sender, items, handler):
            lists.append((sender, items))

        signals.post_get_list.connect(handle_post_get_list)

        user = models.User(name="Bernardo Heynemann", email="foo@bar.com")
        user.save()
        team = models.Team(name="test-team", users=[user])
        team.save()

        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users' % str(team.id)),
        )

        expect(response.code).to_equal(200)
        expect(lists).to_length(1)
        expect(lists[0][0]).to_equal(models.User)
        expect(lists[0][1]).to_length(1)
        expect(lists[0][1][0].slug).to_equal(user.slug)

    @testing.gen_test
    def test_can_save_parent_with_child(self):
        response = yield self.http_client.fetch(
            self.get_url('/parent/'),
            method='POST',
            body='name=Bernardo%20Heynemann&child.first_name=Rodrigo&child.last_name=Lucena'
        )

        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')
        expect(response.headers).to_include('X-Created-Id')
        expect(response.headers).to_include('location')

        parent = models.Parent.objects.get(id=response.headers['X-Created-Id'])
        expect(parent.name).to_equal('Bernardo Heynemann')
        expect(parent.child).not_to_be_null()
        expect(parent.child.first_name).to_equal('Rodrigo')
        expect(parent.child.last_name).to_equal('Lucena')

    @testing.gen_test
    def test_can_save_parent_with_grandchild(self):
        response = yield self.http_client.fetch(
            self.get_url('/parent/'),
            method='POST',
            body='name=Bernardo%20Heynemann'
            '&child.first_name=Rodrigo'
            '&child.last_name=Lucena'
            '&child.child.first_name=Polo'
            '&child.child.last_name=Norte'
        )

        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')
        expect(response.headers).to_include('X-Created-Id')
        expect(response.headers).to_include('location')

        parent = models.Parent.objects.get(id=response.headers['X-Created-Id'])
        expect(parent.name).to_equal('Bernardo Heynemann')

        expect(parent.child).not_to_be_null()
        expect(parent.child.first_name).to_equal('Rodrigo')
        expect(parent.child.last_name).to_equal('Lucena')

        expect(parent.child.child).not_to_be_null()
        expect(parent.child.child.first_name).to_equal('Polo')
        expect(parent.child.child.last_name).to_equal('Norte')

    @testing.gen_test
    def test_can_update_grandchild(self):
        parent = models.Parent.objects.create(name="test-user")

        response = yield self.http_client.fetch(
            self.get_url('/parent/%s/' % str(parent.id)),
            method='PUT',
            body='child.first_name=Rodrigo'
            '&child.last_name=Lucena'
            '&child.child.first_name=Polo'
            '&child.child.last_name=Norte'
        )

        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        loaded = models.Parent.objects.get(id=parent.id)

        expect(loaded.child).not_to_be_null()
        expect(loaded.child.first_name).to_equal('Rodrigo')
        expect(loaded.child.last_name).to_equal('Lucena')

        expect(loaded.child.child).not_to_be_null()
        expect(loaded.child.child.first_name).to_equal('Polo')
        expect(loaded.child.child.last_name).to_equal('Norte')

    @testing.gen_test
    def test_can_save_parent_then_child(self):
        response = yield self.http_client.fetch(
            self.get_url('/parent/'),
            method='POST',
            body='name=Bernardo%20Heynemann'
        )

        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')
        expect(response.headers).to_include('X-Created-Id')
        pk = response.headers['X-Created-Id']

        response = yield self.http_client.fetch(
            self.get_url('/parent/%s/child/' % pk),
            method='POST',
            body='first_name=Rodrigo&last_name=Lucena'
        )

        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        parent = models.Parent.objects.get(id=pk)
        expect(parent.name).to_equal('Bernardo Heynemann')
        expect(parent.child).not_to_be_null()
        expect(parent.child.first_name).to_equal('Rodrigo')
        expect(parent.child.last_name).to_equal('Lucena')

    @testing.gen_test
    def test_can_delete_child_in_parent(self):
        child = models.Child(first_name='Foo', last_name='Bar')
        parent = models.Parent(name='Parent Name')
        parent.child = child
        parent.save()

        response = yield self.http_client.fetch(
            self.get_url('/parent/%s/child/' % (str(parent.id))),
            method='DELETE'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        parent.reload()
        expect(parent.child).to_be_null()

    @testing.gen_test
    def test_can_save_user_team(self):
        team = models.Team.objects.create(name="test-team")
        user = models.User(name="Bernardo Heynemann", email="foo@bar.com")
        user.save()

        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users/' % str(team.id)),
            method='POST',
            body='users[]=%s' % str(user.id)
        )

        team.reload()
        expect(response.code).to_equal(200)
        expect(team.users).to_length(1)
        expect(team.users[0].id).to_equal(user.id)

    @testing.gen_test
    def test_can_get_user_in_team(self):
        user = fix.UserFactory.create()
        team = models.Team.objects.create(name="test-team", users=[user])
        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users/%s' % (team.id, user.id)),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj['email']).to_equal(user.email)
        expect(obj['name']).to_equal(user.name)
        expect(obj['slug']).to_equal(user.slug)

    @testing.gen_test
    def test_can_get_users_list_in_team(self):
        user = models.User(name='Bernardo Heynemann', email='heynemann@gmail.com')
        user.save()
        user2 = models.User(name='Rafael Floriano', email='rflorianobr@gmail.com')
        user2.save()
        team = models.Team.objects.create(name="test-team", users=[user, user2])

        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users/' % str(team.id)),
        )

        expect(response.code).to_equal(200)
        expect(response.body).not_to_be_empty()

        obj = utils.loads(response.body)
        expect(obj).to_length(2)
        expect(obj[0]['name']).to_equal('Bernardo Heynemann')
        expect(obj[0]['email']).to_equal('heynemann@gmail.com')
        expect(obj[1]['name']).to_equal('Rafael Floriano')
        expect(obj[1]['email']).to_equal('rflorianobr@gmail.com')

    @testing.gen_test
    def test_can_get_addresses_for_user_in_team(self):
        address = models.Address(street='Somewhere Else')
        address.save()
        user = models.User(name='Bernardo Heynemann', email='heynemann@gmail.com', addresses=[address])
        user.save()
        team = models.Team(name="test-team", users=[user])
        team.save()

        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users/%s/addresses' % (str(team.id), str(user.id))),
        )

        expect(response.code).to_equal(200)
        expect(response.body).not_to_be_empty()

        obj = utils.loads(response.body)
        expect(obj).to_length(1)
        expect(obj[0]['street']).to_equal(address.street)

    @testing.gen_test
    def test_can_update_addresses_for_user_in_team(self):
        address = models.Address(street='Somewhere Else')
        address.save()
        user = fix.UserFactory.create(addresses=[address])
        team = models.Team.objects.create(name="test-team", users=[user])

        # can't update reference field
        err = expect.error_to_happen(HTTPError)
        with err:
            yield self.http_client.fetch(
                self.get_url('/team/%s/users/%s/addresses/%s' % (str(team.id), str(user.id), str(address.id))),
                method='PUT',
                headers={
                    "Content-Type": "application/x-www-form-urlencoded"
                },
                body='street=Somewhere'
            )
        expect(err.error.code).to_equal(400)

    @testing.gen_test
    def test_can_delete_user_in_team(self):
        user = fix.UserFactory.create()
        team = models.Team.objects.create(name="test-team", users=[user])
        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users/%s' % (str(team.id), str(user.id))),
            method='DELETE'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        user.reload()
        team.reload()
        expect(user).not_to_be_null()
        expect(team.users).to_be_empty()

    @testing.gen_test
    def test_can_delete_addresses_for_user_in_team(self):
        address = models.Address(street='Somewhere Else')
        address.save()
        user = models.User(name='Bernardo Heynemann', email='heynemann@gmail.com', addresses=[address])
        user.save()
        team = models.Team(name="test-team", users=[user])
        team.save()

        response = yield self.http_client.fetch(
            self.get_url('/team/%s/users/%s/addresses/%s' % (str(team.id), str(user.id), str(address.id))),
            method='DELETE'
        )

        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        user.reload()
        team.reload()
        expect(user).not_to_be_null()
        expect(user.addresses).to_be_empty()

    @testing.gen_test
    def test_can_save_child_in_parent2(self):
        parent = models.Parent2.objects.create(name="test-parent")

        response = yield self.http_client.fetch(
            self.get_url('/parent2/%s/children' % str(parent.id)),
            method='POST',
            body='first_name=Rodrigo&last_name=Lucena',
        )

        parent.reload()
        expect(response.code).to_equal(200)
        expect(parent.children).to_length(1)

    @testing.gen_test
    def test_can_get_child_in_parent2(self):
        child = models.Child(first_name='Foo', last_name='Bar')
        parent = models.Parent2.objects.create(name="test-team", children=[child])
        response = yield self.http_client.fetch(
            self.get_url('/parent2/%s/children/Foo' % (parent.id)),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj['first_name']).to_equal('Foo')
        expect(obj['last_name']).to_equal('Bar')

    @testing.gen_test
    def test_can_get_list_of_children_in_parent2(self):
        child = models.Child(first_name='Foo', last_name='Bar')
        parent = models.Parent2.objects.create(name="test-team", children=[child])
        response = yield self.http_client.fetch(
            self.get_url('/parent2/%s/children' % (parent.id)),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj).to_length(1)

    @testing.gen_test
    def test_can_update_child_in_parent2(self):
        child = models.Child(first_name='Foo', last_name='Bar')
        parent = models.Parent2.objects.create(name="test-team", children=[child])
        response = yield self.http_client.fetch(
            self.get_url('/parent2/%s/children/Foo' % (parent.id)),
            method='PUT',
            headers={
                "Content-Type": "application/x-www-form-urlencoded"
            },
            body='first_name=Rafael&last_name=Floriano'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        parent.reload()
        expect(parent.children[0].first_name).to_equal('Rafael')
        expect(parent.children[0].last_name).to_equal('Floriano')

    @testing.gen_test
    def test_can_delete_chold_of_user_in_team(self):
        child = models.Child(first_name='Foo', last_name='Bar')
        parent = models.Parent2.objects.create(name="test-team", children=[child])
        response = yield self.http_client.fetch(
            self.get_url('/parent2/%s/children/Foo' % (parent.id)),
            method='DELETE'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        parent.reload()
        expect(parent).not_to_be_null()
        expect(parent.children).to_be_empty()

    @testing.gen_test
    def test_can_save_student_and_person(self):
        user = models.Person.objects.create(name="Bernardo")
        student = models.Student.objects.create(code="foo")

        response = yield self.http_client.fetch(
            self.get_url('/student/%s/person/' % str(student.id)),
            method='POST',
            body='person[]=%s' % user.id
        )

        student.reload()
        expect(response.code).to_equal(200)
        expect(student.person.name).to_equal('Bernardo')

    @testing.gen_test
    def test_can_get_person_in_student(self):
        person = models.Person.objects.create(name="test-student")
        student = models.Student.objects.create(code="foo", person=person)
        response = yield self.http_client.fetch(
            self.get_url('/student/%s/person/%s' % (student.id, person.id)),
        )
        expect(response.code).to_equal(200)
        obj = load_json(response.body)
        expect(obj['name']).to_equal(person.name)

        response = yield self.http_client.fetch(
            self.get_url('/student/%s/person' % (student.id)),
        )

        expect(response.code).to_equal(200)
        expect(response.body).not_to_be_empty()

        obj = load_json(response.body)
        expect(obj['name']).to_equal(person.name)

    @testing.gen_test
    def test_can_update_person_in_student(self):
        person = models.Person.objects.create(name="test-student")
        student = models.Student.objects.create(code="foo", person=person)
        response = yield self.http_client.fetch(
            self.get_url('/student/%s/person/%s' % (student.id, person.id)),
            method='PUT',
            headers={
                "Content-Type": "application/x-www-form-urlencoded"
            },
            body='name=Rafael'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        person.reload()
        expect(person.name).to_equal('Rafael')

    @testing.gen_test
    def test_can_delete_person_in_student(self):
        person = models.Person.objects.create(name="test-student")
        student = models.Student.objects.create(code="foo", person=person)
        response = yield self.http_client.fetch(
            self.get_url('/student/%s/person/%s' % (student.id, person.id)),
            method='DELETE'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('OK')

        person.reload()
        student.reload()
        expect(person).not_to_be_null()
        expect(student.person).to_be_null()

    def test_can_get_tree_for_single_node(self):
        class Root(mongoengine.Document):
            prop = mongoengine.StringField()
            meta = {'collection': 'root_collection'}

        root_node = me.MongoEngineProvider.get_tree(Root)

        expect(root_node.name).to_equal('Root')
        expect(root_node.slug).to_equal('root')
        expect(root_node.target_name).to_equal('root_collection')
        expect(root_node.model_type).to_equal(Root)
        expect(root_node.is_multiple).to_be_false()
        expect(root_node.allows_create_on_associate).to_be_false()
        expect(root_node.children).to_length(2)
        expect(root_node.required_children).to_be_empty()

        child_node = root_node.children['id']
        expect(child_node.name).to_equal('id')
        expect(child_node.slug).to_equal('id')
        expect(child_node.target_name).to_equal('_id')
        expect(child_node.model_type).to_equal(None)
        expect(child_node.is_multiple).to_be_false()
        expect(child_node.allows_create_on_associate).to_be_false()
        expect(child_node.children).to_be_empty()
        expect(child_node.required_children).to_be_empty()

        child_node = root_node.children['prop']
        expect(child_node.name).to_equal('prop')
        expect(child_node.slug).to_equal('prop')
        expect(child_node.target_name).to_equal('prop')
        expect(child_node.model_type).to_equal(None)
        expect(child_node.is_multiple).to_be_false()
        expect(child_node.allows_create_on_associate).to_be_false()
        expect(child_node.children).to_be_empty()
        expect(child_node.required_children).to_be_empty()

    def test_can_get_tree_for_embedded_document(self):
        class Embedded(mongoengine.EmbeddedDocument):
            name = mongoengine.StringField()
            meta = {'collection': 'embedded_collection'}

        class Root(mongoengine.Document):
            prop = mongoengine.EmbeddedDocumentField(Embedded)
            meta = {'collection': 'root_collection'}

        root_node = me.MongoEngineProvider.get_tree(Root)

        child_node = root_node.children['prop']
        expect(child_node.name).to_equal('prop')
        expect(child_node.slug).to_equal('prop')
        expect(child_node.target_name).to_equal('prop')
        expect(child_node.model_type).to_equal(Embedded)
        expect(child_node.is_multiple).to_be_false()
        expect(child_node.allows_create_on_associate).to_be_true()
        expect(child_node.children).to_length(1)
        expect(child_node.required_children).to_be_empty()

        embedded_doc = child_node.children['name']
        expect(embedded_doc.name).to_equal('name')
        expect(embedded_doc.slug).to_equal('name')
        expect(embedded_doc.target_name).to_equal('name')
        expect(embedded_doc.model_type).to_be_null()
        expect(embedded_doc.is_multiple).to_be_false()
        expect(embedded_doc.allows_create_on_associate).to_be_false()
        expect(embedded_doc.children).to_be_empty()
        expect(embedded_doc.required_children).to_be_empty()

    def test_can_get_tree_for_reference_fields(self):
        class Reference(mongoengine.Document):
            name = mongoengine.StringField()
            meta = {'collection': 'reference_collection'}

        class Root(mongoengine.Document):
            prop = mongoengine.ReferenceField(Reference)
            meta = {'collection': 'root_collection'}

        root_node = me.MongoEngineProvider.get_tree(Root)

        child_node = root_node.children['prop']
        expect(child_node.name).to_equal('prop')
        expect(child_node.slug).to_equal('prop')
        expect(child_node.target_name).to_equal('prop')
        expect(child_node.model_type).to_equal(Reference)
        expect(child_node.is_multiple).to_be_false()
        expect(child_node.allows_create_on_associate).to_be_false()
        expect(child_node.children).to_length(2)
        expect(child_node.required_children).to_be_empty()

        reference_doc = child_node.children['name']
        expect(reference_doc.name).to_equal('name')
        expect(reference_doc.slug).to_equal('name')
        expect(reference_doc.target_name).to_equal('name')
        expect(reference_doc.model_type).to_be_null()
        expect(reference_doc.is_multiple).to_be_false()
        expect(reference_doc.allows_create_on_associate).to_be_false()
        expect(reference_doc.children).to_be_empty()
        expect(reference_doc.required_children).to_be_empty()

    def test_can_get_tree_for_lists_of_reference(self):
        class Reference(mongoengine.Document):
            name = mongoengine.StringField()
            meta = {'collection': 'reference_collection'}

        class Root(mongoengine.Document):
            prop = mongoengine.ListField(mongoengine.ReferenceField(Reference))
            meta = {'collection': 'root_collection'}

        root_node = me.MongoEngineProvider.get_tree(Root)

        child_node = root_node.children['prop']
        expect(child_node.name).to_equal('prop')
        expect(child_node.slug).to_equal('prop')
        expect(child_node.target_name).to_equal('prop')
        expect(child_node.model_type).to_equal(Reference)
        expect(child_node.is_multiple).to_be_true()
        expect(child_node.allows_create_on_associate).to_be_false()
        expect(child_node.children).to_length(2)
        expect(child_node.required_children).to_be_empty()

        reference_doc = child_node.children['name']
        expect(reference_doc.name).to_equal('name')
        expect(reference_doc.slug).to_equal('name')
        expect(reference_doc.target_name).to_equal('name')
        expect(reference_doc.model_type).to_be_null()
        expect(reference_doc.is_multiple).to_be_false()
        expect(reference_doc.allows_create_on_associate).to_be_false()
        expect(reference_doc.children).to_be_empty()
        expect(reference_doc.required_children).to_be_empty()

    def test_can_get_tree_for_lists_of_embedded_document(self):
        class Embedded(mongoengine.EmbeddedDocument):
            name = mongoengine.StringField()
            meta = {'collection': 'embedded_collection'}

        class Root(mongoengine.Document):
            prop = mongoengine.ListField(
                mongoengine.EmbeddedDocumentField(Embedded)
            )
            meta = {'collection': 'root_collection'}

        root_node = me.MongoEngineProvider.get_tree(Root)

        child_node = root_node.children['prop']
        expect(child_node.name).to_equal('prop')
        expect(child_node.slug).to_equal('prop')
        expect(child_node.target_name).to_equal('prop')
        expect(child_node.model_type).to_equal(Embedded)
        expect(child_node.is_multiple).to_be_true()
        expect(child_node.allows_create_on_associate).to_be_true()
        expect(child_node.children).to_length(1)
        expect(child_node.required_children).to_be_empty()

        embedded_doc = child_node.children['name']
        expect(embedded_doc.name).to_equal('name')
        expect(embedded_doc.slug).to_equal('name')
        expect(embedded_doc.target_name).to_equal('name')
        expect(embedded_doc.model_type).to_be_null()
        expect(embedded_doc.is_multiple).to_be_false()
        expect(embedded_doc.allows_create_on_associate).to_be_false()
        expect(embedded_doc.children).to_be_empty()
        expect(embedded_doc.required_children).to_be_empty()

    @testing.gen_test
    def test_can_get_user_list_with_custom_queryset(self):
        models.CustomQuerySet.objects.delete()
        user = models.CustomQuerySet(prop="Bernardo Heynemann")
        user.save()

        user = models.CustomQuerySet(prop="Rafael Floriano")
        user.save()

        response = yield self.http_client.fetch(
            self.get_url('/custom_query_set'),
        )

        expect(response.code).to_equal(200)
        expect(response.body).not_to_be_empty()

        obj = load_json(response.body)
        expect(obj).to_length(1)

        expect(obj[0]['prop']).to_equal('Bernardo Heynemann')

    @testing.gen_test
    def test_can_get_user_instance_with_custom_queryset(self):
        models.CustomQuerySet.objects.delete()
        user = models.CustomQuerySet(prop="Bernardo Heynemann")
        user.save()

        user = models.CustomQuerySet(prop="Rafael Floriano")
        user.save()

        err = expect.error_to_happen(HTTPError)
        with err:
            yield self.http_client.fetch(
                self.get_url('/custom_query_set/%s' % user.id),
            )
        expect(err.error.code).to_equal(404)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com


from setuptools import setup, find_packages
from bzz import __version__

tests_require = [
    'mock',
    'nose',
    'coverage',
    'yanc',
    'preggy',
    'tox',
    'factory-boy',
    'ipdb',
    'coveralls',
    'markupsafe',
    'sphinx',
    'mongoengine',
    'nose-focus',
    'sphinx_rtd_theme',
]

setup(
    name='bzz',
    version=__version__,
    description='bzz is a Rest API framework for working bees.',
    long_description='''
bzz is a Rest API framework for working bees.
''',
    keywords='api rest tornado hal',
    author='Bernardo Heynemann',
    author_email='heynemann@gmail.com',
    url='https://github.com/heynemann/bzz',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: Unix',
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: Implementation :: PyPy",
        'Operating System :: OS Independent',
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'awesome-slugify',
        'cow-framework>=1.0.0',
        'blinker',
        'pyjwt',
        'six',
    ],
    extras_require={
        'tests': tests_require,
    },
    entry_points={
        'console_scripts': [
            # add cli scripts here in this form:
            # 'bzz=bzz.cli:main',
        ],
    },
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com

import cow.server as server
from preggy import expect
import tornado.testing as testing
import derpconf.config as config
from tornado.httpclient import HTTPError

import tests.base as base
import bzz


class TestServer(server.Server):

    def get_handlers(self):
        routes = bzz.MockHive.routes_for([
            ('GET', '/much/api', dict(body='much api')),
            ('POST', '/much/api'),
            ('*', '/much/match', dict(body='such match')),
            ('*', r'/such/.*', dict(body='such match')),
            ('GET', '/much/error', dict(body='WOW', status=404)),
            ('GET', '/much/authentication', dict(body='WOW', cookies={'super': 'cow'})),
            ('GET', '/much/function', dict(body=lambda x: x.method)),
            ('GET', '/much/come/first', dict(body='FIRST')),
            ('*', '/much/come/first', dict(body='NOT FIRTH')),
        ])
        return routes


class TestMockedRoutes(base.ApiTestCase):
    def get_config(self):
        return {}

    def get_server(self):
        cfg = config.Config(**self.get_config())
        self.server = TestServer(config=cfg)
        return self.server

    @testing.gen_test
    def test_can_get_exactly_match(self):
        response = yield self.http_client.fetch(
            self.get_url('/much/api'),
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('much api')

    @testing.gen_test
    def test_can_post_exactly_match(self):
        response = yield self.http_client.fetch(
            self.get_url('/much/api'),
            method='POST',
            body='much anything'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('')

    @testing.gen_test
    def test_can_get_method_wildcard(self):
        response = yield self.http_client.fetch(
            self.get_url('/much/match'),
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('such match')

    @testing.gen_test
    def test_can_post_method_wildcard(self):
        response = yield self.http_client.fetch(
            self.get_url('/much/match'),
            method='POST',
            body='much wow'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('such match')

    @testing.gen_test
    def test_can_get_url_regex(self):
        response = yield self.http_client.fetch(
            self.get_url('/such/match'),
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('such match')

    @testing.gen_test
    def test_can_post_url_regex(self):
        response = yield self.http_client.fetch(
            self.get_url('/such/match'),
            method='POST',
            body='such magic'
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('such match')

    @testing.gen_test
    def test_can_get_cookies(self):
        response = yield self.http_client.fetch(
            self.get_url('/much/authentication')
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('WOW')
        expect(response.headers['Set-Cookie']).to_equal('super=cow; Path=/')

    @testing.gen_test
    def test_can_set_error(self):
        err = expect.error_to_happen(HTTPError)
        with err:
            yield self.http_client.fetch(
                self.get_url('/much/error'),
                method='GET'
            )
        expect(err).to_have_an_error_message_of('HTTP 404: Not Found')

    @testing.gen_test
    def test_cant_delete_exactly_match(self):
        err = expect.error_to_happen(HTTPError)
        with err:
            yield self.http_client.fetch(
                self.get_url('/much/api'),
                method='DELETE'
            )
        expect(err).to_have_an_error_message_of('HTTP 405: Method Not Allowed')

    @testing.gen_test
    def test_body_can_be_a_function(self):
        response = yield self.http_client.fetch(
            self.get_url('/much/function')
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('GET')

    @testing.gen_test
    def test_should_respect_definition_order(self):
        response = yield self.http_client.fetch(
            self.get_url('/much/come/first')
        )
        expect(response.code).to_equal(200)
        expect(response.body).to_equal('FIRST')



#!/usr/bin/python
# -*- coding: utf-8 -*-

# This file is part of bzz.
# https://github.com/heynemann/bzz

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2014 Bernardo Heynemann heynemann@gmail.com

from preggy import expect

import bzz.core as core
import tests.base as base


class MetaModelsTestCase(base.TestCase):
    def test_can_get_property_by_path_inside_node(self):
        node = core.Node('test')
        node.model_type = 123

        inner = core.Node('inner')
        inner.model_type = 456

        node.children[inner.name] = inner

        found = node.find_by_path('inner')
        expect(found).to_equal(inner)

        innerer = core.Node('innerer')  # like that's a word
        inner.children[innerer.name] = innerer

        found = node.find_by_path('inner.innerer')
        expect(found).to_equal(innerer)
